// ================================================================
// GenomeMixer    (c) 2003, Alex Williams
// All rights reserved.
// Send email correspondence to Robert W. Williams <rwilliam@nb.utmem.edu>
// at the University of Tennesse Health Science Center

// This project was funded jointly by the National Institute of Mental Health, National Institute on Drug Abuse,
// and the National Science Foundation (award P20-MH 62009 to KFM and RW)

// This source code may be used according to the terms of the GNU General Public License (GPL).
// See www.gnu.org for specifics. Essentially, the source may be freely used as long as the source code
// for whatever program the source is also made freely available. If you want to use this source
// in a closed-source program, you should email us for licensing options.

// Disclaimer: This code comes with no warranty at all, and there is no guarantee that it will
// work as claimed in the documentation. Hopefully, it will be useful anyway.

// Note that GenomeMixer makes use of the Trolltech Qt toolkit for its window-managing needs.
// There is very little use of Qt data structures (aside from QString) in the actual processing code,
// but **you will need to have Qt installed in order to compile this code.** Check <www.trolltech.com>
// for details; there should be a free GPL-software-only version of Qt available on the Trolltech
// web site.

// Any redistributions of this software in code or binary form must include the
// above copyright notice, licensing information, disclaimer, and this notice itself.
// Basically, all the comments at the top of this file must be retained.
// ================================================================


#include "CellData.h"
#include "MouseIO.h"
#include "PurebredData.h"

#include <assert.h>
#include <string>
#include <qstring.h>


CellData::CellData(bool comment, bool toExport, int rand, int fem, int male) {  // Constructor
    myValid = true;     
    myCommentedOut = comment;
    myToBeExported = toExport;
  
    myRandomAssignSex = rand; // <-- number of progeny to assign a sex to randomly
    myMinFemales = fem;
    myMinMales = male;
      // Below: these are just set to their defaults
    myMotherGen = 0;
    myMotherCage = 0;	
    myFatherGen = 0;
    myFatherCage = 0;
    myMotherPurebredIndex = -1;
    myFatherPurebredIndex = -1;  
}


CellData::CellData(CellData* other) { // copy-ish constructor
    this->overwriteWithDataFrom(other);
}

CellData::CellData(void) { // Constructor
  myCommentedOut = false;
  myToBeExported = false;
  myValid = false;
  
  myRandomAssignSex = 0;
  myMinFemales = 1;
  myMinMales = 1;
  
  myMotherGen = 0;
  myMotherCage = 0;	
  myFatherGen = 0;
  myFatherCage = 0;       	
  
  myMotherPurebredIndex = -1;
  myFatherPurebredIndex = -1;  
}


void CellData::overwriteWithDataFrom(CellData* other) { // copies the data into this CellData
    assert(other != NULL);
    
    this->myValid                           = other->myValid;
    this->myCommentedOut          = other->myCommentedOut;
    this->myToBeExported          = other->myToBeExported;	
    
    this->myRandomAssignSex    = other->myRandomAssignSex;
    this->myMinFemales            = other->myMinFemales;
    this->myMinMales              = other->myMinMales;  
    
    this->myMotherGen             = other->myMotherGen;
    this->myMotherCage            = other->myMotherCage;	
    this->myFatherGen             = other->myFatherGen;
    this->myFatherCage            = other->myFatherCage;  
    this->myMotherPurebredIndex   = other->myMotherPurebredIndex;
    this->myFatherPurebredIndex   = other->myFatherPurebredIndex;  
}


string CellData::toString(PurebredData *pure) {
  // Returns a string representation of the data (or some of it, at least) in this CellData object    
    if (!myValid) {
	return QString::null;
    } else {
	return (
		(myCommentedOut   ? QString("#")    : QString::null)
		+ (myToBeExported ? QString("exp ") : QString::null)
		+ QString(motherString(pure))
		+ QString(" x ")
		+ QString(fatherString(pure))
		+  (QString(" r=") + QString::number(myRandomAssignSex))
		+  (QString(" f=") + QString::number(myMinFemales))
		+  (QString(" m=") + QString::number(myMinMales))
		);
    }
}



string CellData::motherString(PurebredData *pure) {
  if (motherIsPurebred()) {
    if (myMotherPurebredIndex < 0) {
      cout << "Error: mother purebred index less than 0, but somehow it is supposed to be a purebred." << endl;
      return string("Bad name: Error 1 - maternal");
    }
    return (string("p(") + *(pure->getPureNamePtr(myMotherPurebredIndex)) + string(")"));
  } else {
    if (myMotherCage < 0 || myMotherGen < 0) {
      cout << "Bad name... error." << endl;
      return string("Bad Name: Error");
    }
    return QString(MouseIO::generationNumToLetters(myMotherGen)) + QString::number(myMotherCage+1);
  }
}

string CellData::fatherString(PurebredData *pure) {
  if (fatherIsPurebred()) {
    if (myFatherPurebredIndex < 0) {
      cout << "Error: father purebred index less than 0, but somehow it is supposed to be a purebred." << endl;
      return string("Bad name: Error 2 - paternal");
    }
    return (string("p(") + *(pure->getPureNamePtr(myFatherPurebredIndex)) + string(")"));
  } else {
    if (myFatherCage < 0 || myFatherGen < 0) {
      cout << "Bad name... error." << endl;
      return string("Bad Name: Error 3 - paternal");
    }    
    return QString(MouseIO::generationNumToLetters(myFatherGen)) + QString::number(myFatherCage+1);
  }
}


void CellData::setCommentedOut(bool newValue) {
  myCommentedOut = newValue;
  myValid = true;
}

void CellData::setToBeExported(bool newValue) {
  myToBeExported = newValue;
  myValid = true;
}

bool CellData::commentedOut(void) {
  return myCommentedOut;
}

bool CellData::toBeExported(void) {
  return myToBeExported;
}


int CellData::randomSex(void) {
  return myRandomAssignSex;
}
int CellData::minFemales(void) {
  return myMinFemales;
}
int CellData::minMales(void) {
  return myMinMales;
}

void CellData::setRandomSex(int numRandom) {
  this->myRandomAssignSex = numRandom;
  myValid = true;
}
void CellData::setMinFemales(int newMinFemales) {
  myMinFemales = newMinFemales;
  myValid = true;
}
void CellData::setMinMales(int newMinMales) {
  myMinMales = newMinMales;
  myValid = true;
}


void CellData::setMother(int cageIndex, int genIndex) {
  myMotherPurebredIndex = -1;  // negative numbers mean "not a purebred"
  myMotherCage = cageIndex;  
  myMotherGen  = genIndex;
  myValid = true;
}

void CellData::setFather(int cageIndex, int genIndex) {
  myFatherPurebredIndex = -1;  // negative numbers mean "not a purebred"
  myFatherCage = cageIndex;
  myFatherGen  = genIndex;
  myValid = true;
}

void CellData::setMotherPurebred(int pureIndexFem) {
  assert(pureIndexFem >= 0);
  myMotherCage = -1;
  myMotherGen  = -1;
  myMotherPurebredIndex = pureIndexFem;
  myValid = true;
}
void CellData::setFatherPurebred(int pureIndexMale) {
  assert(pureIndexMale >= 0);
  myFatherCage = -1;
  myFatherGen  = -1;
  myFatherPurebredIndex = pureIndexMale;
  myValid = true;
}


int CellData::motherCage(void) {
  return myMotherCage;
}
int CellData::motherGen(void) {
  return myMotherGen;
}

int CellData::fatherCage(void) {
  return myFatherCage;
}
int CellData::fatherGen(void) {
  return myFatherGen;
}

bool CellData::motherIsPurebred(void) {
  return (myMotherPurebredIndex >= 0); // negative numbers mean "not a purebred"
}
bool CellData::fatherIsPurebred(void) {
  return (myFatherPurebredIndex >= 0); // negative numbers mean "not a purebred"
}
int CellData::motherPurebredIndex(void) {
  assert(motherIsPurebred());
  return myMotherPurebredIndex;
}
int CellData::fatherPurebredIndex(void) {
  assert(fatherIsPurebred());
  return myFatherPurebredIndex;
}






